/* -*- tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
import lejos.nxt.*;

/**
 * This class contains all the PID magic happening that makes Marvin keep its balance.
 * The balancing module reads the angles and the angle velocities from the gyro and
 * motors, and uses these values (weighted) to feed to the PID control calculation
 * which again produces the new power to apply to the motors.
 * 
 * @author Bent Bisballe Nyeng, Kasper Sohn and Johnny Rieper
 * @version January 2009
 */
public class BalanceControl extends Thread
{
  // The PID control parameters
  private final double Kp = 1.0;
  private final double Ki = 0.2;
  private final double Kd = 0.2;

  // The error contributions.
  private final double K_psi = 31.9978; // Gyro angle weight
  private final double K_phi = 0.8703; // Motor angle weight
  private final double K_psidot = 0.6; // Gyro angle velocity weight
  private final double K_phidot = 0.02; // Motor angle velocity weight

  private Ctrl ctrl;

  /**
   * BalanceControl constructor.
   * @param ctrl The motor control parameters.
   */
  public BalanceControl(Ctrl ctrl)
  {
    this.ctrl = ctrl;
    setDaemon(true);
  }

  public void run()
  {	
    MotorControl motors = new MotorControl(Motor.C, Motor.B);
    GyroscopeSensor gyro = new GyroscopeSensor(SensorPort.S4);

    double int_error = 0.0;
    double prev_error = 0.0;

    while(true) {
	    double Psi = gyro.getAngle() + ctrl.tiltAngle();
	    double PsiDot = gyro.getAngleVelocity();
	    
	    double Phi = motors.getAngle();
	    double PhiDot = motors.getAngleVelocity();

	    double error = Psi * K_psi + Phi * K_phi + PsiDot * K_psidot + PhiDot * K_phidot;

	    int_error += error;	// Integral Error

	    double deriv_error = error - prev_error; // Derivative Error
	    prev_error = error;

	    double pw =	error * Kp + deriv_error * Kd + int_error * Ki;

	    motors.setPower(pw + ctrl.leftMotorOffset(), pw + ctrl.rightMotorOffset());

	    try { sleep(3); } catch(java.lang.InterruptedException e) { }
    }
  }    
}
