/* -*- tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
import java.util.ArrayList;

/**
 * A platform for a behavior based system with a simple
 * suppression mechanism. An actual behavior is defined by extending 
 * the Behavior class. Each behavior is a single thread of control.
 * It has access to control the Marvin by means of the control methods
 * (forward, backward, left and right).
 * However, access can be suppressed so that the motor commands
 * will not be performed while the behavior is suppressed.
 * 
 * @author Bent Bisballe Nyeng, Kasper Sohn and Johnny Rieper
 * @version January 2009
 */
public class Behavior extends Thread 
{
  private ArrayList behaviors;;
  private boolean suppressed;
  private Ctrl ctrl;

  private final double turnPower = 200;
  private final double tiltPower = 0.06;

  /**
   * Bahavior constructor.
   * @param behaviors List of lower priority behaviors.
   * @param ctrl The motor control parameters.
   */
  public Behavior(ArrayList behaviors, Ctrl ctrl)
  {
    this.ctrl = ctrl;
    suppressed = false;

    this.setDaemon(true);

    // Clone the Behavior list.
    this.behaviors = new ArrayList();
    for(int i = 0; i < behaviors.size(); i++) {
	    Behavior b = (Behavior)behaviors.get(i);
	    this.behaviors.add(b);
    }
  }

  /**
   * suppressLower suppresses all behaviors in the behavior list.
   */
  public void suppressLower()
  {
    for(int i = 0; i < behaviors.size(); i++) {
	    Behavior b = (Behavior)behaviors.get(i);
	    b.setSuppress(true);
    }
  }

  /**
   * unsuppressLower unsuppresses all behaviors in the behavior list.
   */
  public void unsuppressLower()
  {
    for(int i = 0; i < behaviors.size(); i++) {
	    Behavior b = (Behavior)behaviors.get(i);
	    b.setSuppress(false);
    }
  }

  /**
   * isSuppressed simply returns the value of the variable suppressed, but does it in sync.
   * @return A boolean representing the suppress state of this behavior.
   */
  public synchronized boolean isSuppressed()
  {
    return suppressed;
  }

  /**
   * setSuppress simply sets the value of the suppressed variable, but does it in sync.
   * This method is used to conitrol the suppressed state of this behvaior.
   * @param suppress A boolean containing the new value of suppressed.
   */
  public synchronized void setSuppress(boolean suppress)
  {
    suppressed = suppress;
  }

  /**
   * Instruct Marvin to drive forward for a given period of time.
   * @param period An integer containing the number of milisecond to drive before returning.
   */
  public void forward(int period)
  {
    if(!isSuppressed()) {
	    ctrl.setLeftMotorOffset(0);
	    ctrl.setRightMotorOffset(0);

	    for(int time = 0; time < period; time += 10) {
        try { Thread.sleep(10); } catch(InterruptedException e) { }
        ctrl.setTiltAngle(-tiltPower);
	    }
    }
  }

  /**
   * Instruct Marvin to drive backwards for a given period of time.
   * @param period An integer containing the number of milisecond to drive before returning.
   */
  public void backward(int period)
  {
    if(!isSuppressed()) {
	    ctrl.setLeftMotorOffset(0);
	    ctrl.setRightMotorOffset(0);

	    for(int time = 0; time < period; time += 10) {
        try { Thread.sleep(10); } catch(InterruptedException e) { }
        ctrl.setTiltAngle(tiltPower);
	    }
    }
  }

  /**
   * Instruct Marvin to turn left for a given period of time.
   * @param period An integer containing the number of milisecond to rotate before returning.
   */
  public void left(int period)
  {
    if(!isSuppressed()) {
	    ctrl.setLeftMotorOffset(-turnPower);
	    ctrl.setRightMotorOffset(turnPower);
	    delay(period);
    }
  }

  /**
   * Instruct Marvin to turn right for a given period of time.
   * @param period An integer containing the number of milisecond to rotate before returning.
   */
  public void right(int period)
  {
    if(!isSuppressed()) {
	    ctrl.setLeftMotorOffset(turnPower);
	    ctrl.setRightMotorOffset(-turnPower);
	    delay(period);
    }
  }

  /**
   * Instruct Marvin to stop, and stay stopped for a given period of time.
   * @param period An integer containing the number of milisecond to pause before returning.
   */
  public void stop(int period)
  {
    if(!isSuppressed()) {
	    ctrl.setLeftMotorOffset(0);
	    ctrl.setRightMotorOffset(0);
	    delay(period);
    }
  }

  /**
   * Make this behavior thread sleep for a number of miliseconds.
   * @param time An integer containing the number of miliseconds to sleep.
   */
  public void delay(int time)
  {
    try {
      Thread.sleep(time);
    } catch (Exception e) {
    }
  }
	
  /**
   * Override the run method to define the actual behavior.
   */
  public void run()
  {
  }
}
