/* -*- tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
import lejos.nxt.*;
 
/**
 * This class is an interface to the gyroscope sensor (a sensor that reads angle velocities).
 * 
 * Values can be read as angle velocities, or as integrated values (angles).
 *
 * @author Bent Bisballe Nyeng, Kasper Sohn and Johnny Rieper
 * @version January 2009
 */ 
public class GyroscopeSensor
{
  private SensorPort port;

  // For angle calculation (integration)
  private double angle = 0.0;
  private int lastGetAngleTime = 0;

  private double lastOffset = 595.5; // Experimental offset value
  private final double a = 0.999999; // Weight of older offset values.

  /**
   * The GyroscopeSensor constructor.
   * @param port The SensorPort to which the gyroscope is connected.
   */ 
  public GyroscopeSensor(SensorPort port)
  {
    this.port = port;
  }

  /**
   * Calculate angle offset according to battery level.
   * @return A double containing the current angle offset value.
   */
  private double getAngleOffset()
  {
    double offset = lastOffset * a + (1.0 - a) * (double)port.readValue();
    lastOffset = offset;
    return offset;
  }

  /**
   * Get the angle velocity.
   * @return A double containing the angle change in degrees per second
   */
  public double getAngleVelocity()
  {
    double offset = getAngleOffset();
    return (double)port.readValue() - offset;
  }

  /**
   * Get the calculated angle (angle velocity integrated over time).
   * @return The angle in degrees.
   */
  public double getAngle()
  {
    int now = (int)System.currentTimeMillis();
    int delta_t = now - lastGetAngleTime;
 
    // Make sure we only add to the sum when there has actually 
    // been a previous call (delta_t == now if its the first call).
    if(delta_t != now) {
      angle += getAngleVelocity() * ((double)delta_t / 1000.0);
    }
 
    lastGetAngleTime = now;

    return angle;
  }
}
