/* -*- tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
import lejos.nxt.*;
 
/**
 * A class that controls the motors, and reports angle and angle velocity
 * (using the TACHO counter).
 *
 * @author Bent Bisballe Nyeng, Kasper Sohn and Johnny Rieper
 * @version January 2009
 */
class MotorControl
{
  private Motor leftMotor;
  private Motor rightMotor;

  private double sin_x = 0.0;
  private final double sin_speed = 0.1;
  private final double sin_amp = 20.0;

  /**
   * MotorControl constructor.
   * It resets the TACHO counters of both motors.
   * @param leftMotor The left motor to control.
   * @param rightMotor The right motor to control.
   */
  public MotorControl(Motor leftMotor, Motor rightMotor)
  {
    this.leftMotor = leftMotor;
    this.leftMotor.resetTachoCount();

    this.rightMotor = rightMotor;
    this.rightMotor.resetTachoCount();
  }

  /**
   * setPower is the control method of the motors.
   * It sets the power to the given values, but on top of that applies a dampened smooth
   * sinusoidal movement to make gears in the motors strong up at all times.
   * @param leftPower A double containing the power to set on the left motor. Its value is
   *  in degrees per second, and at has a maximum of approximately 800 (depending on battery level).
   *  Negative values makes the motor turn backwards.
   * @param rightPower A double containing the power to set on the right motor. Its value is
   *  in degrees per second, and at has a maximum of approximately 800 (depending on battery level)
   *  Negative values makes the motor turn backwards.
   */
  public void setPower(double leftPower, double rightPower)
  {
    sin_x += sin_speed;
    int pwl = (int)( leftPower + Math.sin(sin_x) * sin_amp );
    int pwr = (int)( rightPower - Math.sin(sin_x) * sin_amp );

    leftMotor.setSpeed(pwl);
    if(pwl < 0) {	
	    leftMotor.backward();
    } else if(pwl > 0) {
	    leftMotor.forward();
    } else {
	    leftMotor.stop();
    }

    rightMotor.setSpeed(pwr);
    if(pwr < 0) {
	    rightMotor.backward();
    } else if(pwr > 0) {
	    rightMotor.forward();
    } else {
	    rightMotor.stop();
    }
  }

  /**
   * getAngle is used to get the average TACHO angle of the two motors.
   * @return A double containing the average angle of the motors in degrees.
   */
  public double getAngle()
  {
    return ((double)leftMotor.getTachoCount() + (double)rightMotor.getTachoCount()) / 2.0;
  }

  /**
   * getAngleVelocity is used to get the average TACHO angle velocity of the two motors.
   * @return A double containing the average angle velocity of the motors in degrees per second.
   */
  public double getAngleVelocity()
  {
    return ((double)leftMotor.getActualSpeed() + (double)rightMotor.getActualSpeed()) / 2.0;
  }
}